package br.com.bookstore.bean;

import java.util.List;

import br.com.bookstore.cliente.Usuario;
import br.com.bookstore.exceptions.ClienteException;

/**
 * Fachada que concentra o ponto de entrada para todas
 * as operaes que envolvem o gerenciamento de clientes.
 * 
 * @author Adelino Ferreira - bugass@gmail.com
 */
public interface ClienteBeanModel {

	/**
	 * Realiza o cadastro de um novo cliente. O cadastro no deve ser permitido
	 * caso o cliente esteja marcado com o perfil de administrador. Da mesma forma,
	 * o cadastro tambm no deve ser permitido caso j exista um cliente com o mesmo
	 * CPF cadastrado na base de dados da aplicao.
	 * 
	 * @param cliente o Cliente a ser cadastrado.
	 * @throws ClienteException Caso o cliente a ser cadastrado j exista na base de dados, possua alguma
	 * 							informao invlida, ou esteja com o perfil de administrador configurado.
	 */
	public void cadastrarCliente( Usuario cliente ) throws ClienteException;
	
	/**
	 * Exclui um cliente da base de dados da aplicao. No deve ser possvel, via aplicao,
	 * remover um usurio, cujo perfil seja administrador. 
	 * 
	 * @param cliente O cliente a ser removido
	 * @throws ClienteException Caso o cliente a ser removido no exista, ou possua o perfil de administrador.
	 */
	public void removerCliente( Usuario cliente ) throws ClienteException;
	
	/**
	 * Permite modificar as informaes cadastrais de um cliente. No deve ser possvel alterar o CPF
	 * do cliente, nem mudar o seu perfil. As informaes atuais do cliente sero subistitudas pelas 
	 * informaes fornecidas atravs do objeto passado como parmetro.
	 * 
	 * @param cliente O cliente cujas informaes sero modificadas.
	 * @throws ClienteException Caso existam restries nos dados fornecidos atravs do novo objeto passado
	 * 							como parmetro ou o cliente a ser modificado no exista nas bases de dados.
	 */
	public void editarCliente( Usuario cliente ) throws ClienteException;
	
	/**
	 * Obtm uma lista com todos os clientes cadastrados na aplicao. Esta lista no deve conter usurios
	 * marcados como administradores.
	 * 
	 * @return Uma lista com todos os clientes cadastrados, ou uma lista vazia, caso no exista nenhum
	 * 			cliente cadastrado.
	 */
	public List<Usuario> listarClientes();

	/**
	 * Obtm o cliente representado pelo CPF passado como parmetro. Usurios que possuem o perfil
	 * de administrador no so visveis a esta chamada.
	 * 
	 * @param cpf o CPF usado para recuperar o cliente especfico.
	 * @return o cliente cujo CPF foi passado como parmetro, ou null, caso no exista cliente cadastrado
	 * 			com o CPF em questo.
	 */
	public Usuario obterCliente( String cpf ) throws ClienteException ; 
	
	/**
	 * Obtm uma lista com todos os estados reconhecidos pela aplicao.
	 * @return Uma lista com os estados em formato String.
	 */
	public List<String> obterTodosOsEstados();
	
	/**
	 * Obtm todas as cidades referentes ao estado passado como parmetro.
	 * @param estado o estado para o qual estamos procurando a cidade.
	 * @return O conjunto de cidades referentes ao estado, ou uma lista vazia, 
	 * 			caso no haja cidades cadastradas.
	 */
	public List<String> obterCidades( String estado );
	
}
